<?php
/**
 * Styling Panel customize settings.
 *
 * @package    Community_Builder
 * @subpackage Customizer\Panels
 * @copyright  Copyright (c) 2018, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Brajesh Singh
 */

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Styling Panel helper.
 */
class CB_Customize_Panel_Item_Entries {

	/**
	 * Panel Id.
	 *
	 * @var string
	 */
	private $panel = 'cb_item-entries';

	/**
	 * CB_Customize_Panel_Styling constructor.
	 */
	public function __construct() {
		add_filter( 'cb_customizer_sections', array( $this, 'add_sections' ) );
	}

	/**
	 * Add sections for this panel.
	 *
	 * @param array $sections sections.
	 *
	 * @return array
	 */
	public function add_sections( $sections ) {

		$new_sections = $this->get_sections();

		return array_merge( $sections, $new_sections );
	}

	/**
	 * Get all sections for this panel.
	 *
	 * @return array
	 */
	public function get_sections() {

		$panel               = $this->panel;
		$sections = array();

		/**
		 * Main Column
		 */
		$sections['item-list-item-entry'] = array(
			'priority' => 0,
			'panel'    => $panel,
			'title'    => __( 'Item List Entries', 'community-builder' ),
			'options'  => array(

				'item-entries-info-title'             => array(
					'control' => array(
						'control_type' => 'CB_Customize_Control_Info_Title',
						'label'        => __( 'Item List Entries', 'community-builder' ),
						'description'  => __( 'These controls modify loop entries(depending on context posts, page, events etc)', 'community-builder' ),
					),
				),
				'item-list-entry-title-font-settings' => CB_Customize_Setting_Builder::get_typography_settings(
					'item-list-entry-title',
					__( 'Title font settings', 'community-builder' )
				),
				'item-list-entry-title-link-color'         => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Title color', 'community-builder' ),
						'default' => cb_get_default( 'item-list-entry-title-link-color' ),

					)
				),
				'item-list-entry-title-link-hover-color'   => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Title hover color', 'community-builder' ),
						'default' => cb_get_default( 'item-list-entry-title-link-hover-color' ),

					)
				),
				'item-list-entry-meta-font-settings'  => CB_Customize_Setting_Builder::get_typography_settings(
					'item-list-entry-meta',
					__( 'Meta font settings', 'community-builder' )
				),


				'item-list-entry-meta-text-color' => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Meta text color', 'community-builder' ),
						'default' => cb_get_default( 'item-list-entry-meta-text-color' ),
					)
				),
				'item-list-entry-meta-separator-text-color' => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Meta separator color', 'community-builder' ),
						'default' => cb_get_default( 'item-list-entry-meta-separator-text-color' ),
					)
				),

				'item-list-entry-meta-link-color'       => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Meta link color', 'community-builder' ),
						'default' => cb_get_default( 'item-list-entry-meta-link-color' ),

					)
				),
				'item-list-entry-meta-link-hover-color' => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Meta link hover color', 'community-builder' ),
						'default' => cb_get_default( 'item-list-entry-meta-link-hover-color' ),

					)
				),

				'item-list-entry-content-font-settings' => CB_Customize_Setting_Builder::get_typography_settings(
					'item-list-entry-content',
					__( 'Content Font settings', 'community-builder' )
				),

				'item-list-entry-content-text-color'       => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Content text color', 'community-builder' ),
						'default' => cb_get_default( 'item-list-entry-content-text-color' ),

					)
				),
				'item-list-entry-content-link-color'       => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Content link color', 'community-builder' ),
						'default' => cb_get_default( 'item-list-entry-content-link-color' ),

					)
				),
				'item-list-entry-content-link-hover-color' => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Content link hover color', 'community-builder' ),
						'default' => cb_get_default( 'item-list-entry-content-link-hover-color' ),

					)
				),
			),
		);


		/**
		 * Main Column
		 */
		$sections['item-entries-single-item-entry'] = array(
			'priority' => 1,
			'panel'    => $panel,
			'title'    => __( 'Single Entry', 'community-builder' ),
			'options'  => array(
				'item-entries-info-title'             => array(
					'control' => array(
						'control_type' => 'CB_Customize_Control_Info_Title',
						'label'        => __( 'Single Entry', 'community-builder' ),
						'description'  => __( 'These controls modify single entry(depending on context single posts, page etc)', 'community-builder' ),
					),
				),
				'item-entry-title-font-settings' => CB_Customize_Setting_Builder::get_typography_settings(
					'item-entry-title',
					__( 'Title font settings', 'community-builder' )
				),
				'item-entry-title-link-color'         => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Title color', 'community-builder' ),
						'default' => cb_get_default( 'item-entry-title-link-color' ),

					)
				),
				'item-entry-title-link-hover-color'   => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Title hover color', 'community-builder' ),
						'default' => cb_get_default( 'item-entry-title-link-hover-color' ),

					)
				),
				'item-entry-meta-font-settings'  => CB_Customize_Setting_Builder::get_typography_settings(
					'item-entry-meta',
					__( 'Meta font settings', 'community-builder' )
				),


				'item-entry-meta-text-color' => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Meta text color', 'community-builder' ),
						'default' => cb_get_default( 'item-entry-meta-text-color' ),
					)
				),
				'item-entry-meta-separator-text-color' => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Meta separator color', 'community-builder' ),
						'default' => cb_get_default( 'item-entry-meta-separator-text-color' ),
					)
				),

				'item-entry-meta-link-color'       => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Meta link color', 'community-builder' ),
						'default' => cb_get_default( 'item-entry-meta-link-color' ),

					)
				),
				'item-entry-meta-link-hover-color' => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Meta link hover color', 'community-builder' ),
						'default' => cb_get_default( 'item-entry-meta-link-hover-color' ),
					)
				),

				'item-entry-content-font-settings' => CB_Customize_Setting_Builder::get_typography_settings(
					'item-entry-content',
					__( 'Content Font settings', 'community-builder' )
				),

				'item-entry-content-text-color'       => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Content text color', 'community-builder' ),
						'default' => cb_get_default( 'item-entry-content-text-color' ),

					)
				),
				'item-entry-content-link-color'       => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Content link color', 'community-builder' ),
						'default' => cb_get_default( 'item-entry-content-link-color' ),

					)
				),
				'item-entry-content-link-hover-color' => CB_Customize_Setting_Builder::get_color_settings(
					array(
						'label'   => __( 'Content link hover color', 'community-builder' ),
						'default' => cb_get_default( 'item-entry-content-link-hover-color' ),

					)
				),
			),
		);
		$sections = apply_filters( 'cb_customizer_item_entries_sections', $sections );

		return $sections;
	}

} // end of class.

new CB_Customize_Panel_Item_Entries();
